<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xs="http://www.w3.org/2001/XMLSchema"
 xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:func="http://oxygenxml.com/samples/books"
 exclude-result-prefixes="xs xd func"
 xpath-default-namespace="http://oxygenxml.com/samples/books"
 version="2.0">
 <xd:doc scope="stylesheet">
  <xd:desc>
   <xd:p><xd:b>Created on:</xd:b> May 8, 2013</xd:p>
   <xd:p><xd:b>Author:</xd:b> Octavian Nadolu</xd:p>
   <xd:p>Generates an HTML file containing a table with books.</xd:p>
  </xd:desc>
 </xd:doc>
 
 <xd:doc>
  <xd:desc>
   <xd:b>Specifies if the books should be grouped by technology.</xd:b>
  </xd:desc>
 </xd:doc>
 <xsl:param name="groupByTechnology" select="false()"/>
 
 <xd:doc>
  <xd:desc>
   <xd:b>Depending on the value of the parameter an index of the books will be created or
    not.</xd:b>
  </xd:desc>
 </xd:doc>
 <xsl:param name="generateIndex" select="true()"/>
 
 <xd:doc>
  <xd:desc>
   <xd:p>Generates a table with books.</xd:p>
  </xd:desc>
 </xd:doc>
 <xsl:template match="books">
  <html>
   <head>
    <title>Recommended Books</title>
   </head>
   <body style="font-size:0.96em">
    <h1>The list of recommended books</h1>
    
    <!--<xsl:if test="$generateIndex">
     <xsl:copy-of select="func:generateIndex(book)"/>
     </xsl:if>-->
    
    <xsl:choose>
     <xsl:when test="$groupByTechnology">
      <!-- Generates a table for each group of books that have the same technology.-->
      <xsl:for-each-group select="book" group-by="@technology">
       <h2>
        <xsl:value-of select="@technology"/>
       </h2>
       <hr/>
       <!-- A table is create for each group. -->
       <table cellspacing="6px">
        <xsl:apply-templates select="current-group()"/>
       </table>
      </xsl:for-each-group>
     </xsl:when>
     <xsl:otherwise>
      <table cellspacing="6px">
       <!-- Generates a table with books, in the order they are in the input XML document. -->
       <xsl:apply-templates select="
        book[not(title = preceding-sibling::book/title)]"/>
      </table>
     </xsl:otherwise>
    </xsl:choose>
   </body>
  </html>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>
   <xd:p>Generates an index for the given books.</xd:p>
  </xd:desc>
  <xd:param name="books">The books to generate the index for.</xd:param>
 </xd:doc>
 <xsl:function name="func:generateIndex">
  <xsl:param name="books"/>
  <!-- Index  -->
  <xsl:for-each select="$books">
   <xsl:call-template name="createIndexItem">
    <xsl:with-param name="bookAnchor" select="@anchor"/>
    <xsl:with-param name="bookTitle" select="title"/>
    <xsl:with-param name="bookAuthor" select="author"/>
   </xsl:call-template>
  </xsl:for-each>
 </xsl:function>
 
 <xd:doc>
  <xd:desc>
   <xd:p>Generates a link for a book the index.</xd:p>
  </xd:desc>
  <xd:param name="bookAnchor">The anchor of the reference.</xd:param>
  <xd:param name="bookName">The text displayed for the link.</xd:param>
  <xd:param name="bookAuthor">The author of the book.</xd:param>
 </xd:doc>
 <xsl:template name="createIndexItem">
  <xsl:param name="bookAnchor"/>
  <xsl:param name="bookTitle"/>
  <xsl:param name="bookAuthor"/>
  <p>
   <a href="{concat('#', $bookAnchor)}" title="{$bookTitle}">
    <xsl:value-of select="$bookTitle"/>
   </a>
   <i> - by <xsl:value-of select="$bookAuthor"/></i>
  </p>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>
   <xd:p>Generate a row in the table for each book.</xd:p>
  </xd:desc>
 </xd:doc>
 <xsl:template match="book">
  <!-- Book -->
  <tr>
   <td valign="top">
    <!--Image-->
    <xsl:apply-templates select="image"/>
   </td>
   <td>
    <a name="{@anchor}"/>
    <!-- Title with link -->
    <xsl:apply-templates select="title"/>
    
    <!-- Author -->
    <xsl:apply-templates select="author"/>
    
    <!-- Book info -->
    <xsl:apply-templates select="paperback"/>
    <xsl:apply-templates select="publisher"/>
    <xsl:apply-templates select="date"/>
    <xsl:apply-templates select="isbn"/>
    
    <!-- Description -->
    <xsl:apply-templates select="description"/>
    
    <!-- Related books -->
    <p>
     <b>Related books:</b>
    </p>
    <xsl:variable name="currentTechnology" select="@technology"/>
    <xsl:variable name="currentTitle" select="title"/>
    
    <!-- Get all the books that have the same technolology -->                
    <xsl:for-each select="/books/book">
     <xsl:if test="@technology = $currentTechnology">
      <xsl:if test="not(title = preceding-sibling::book/title 
       or title = $currentTitle)">
       <!-- Display the books -->
       <xsl:call-template name="createIndexItem">
        <xsl:with-param name="bookAnchor" select="@anchor"/>
        <xsl:with-param name="bookTitle" select="@anchor"/>
        <xsl:with-param name="bookAuthor" select="author"/>
       </xsl:call-template>
      </xsl:if>
     </xsl:if>
    </xsl:for-each>
    <br/>
   </td>
  </tr>
 </xsl:template>
 
 <xsl:template match="title">
  <!-- Title with link -->
  <xsl:variable name="title" select="."/>
  <xsl:variable name="titleURI" select="parent::node()/@targeturi"/>
  
  <xsl:call-template name="linkTitle">
   <xsl:with-param name="titleUri" select="$titleURI"/>
   <xsl:with-param name="title" select="$title"/>
  </xsl:call-template>
 </xsl:template>
 
 <xsl:template match="author">
  <p>
   <i>by 
    <xsl:value-of select="node()"/>
   </i>
  </p>
 </xsl:template>
 
 <xsl:template match="paperback">
  <b>Paperback: <xsl:value-of select="node()"/> </b>|
 </xsl:template>
 
 <xsl:template match="publisher">
  <b>Publisher: </b><xsl:value-of select="node()"/>
 </xsl:template>
 
 <xsl:function name="func:getDate">
  <xsl:param name="date"/>
  <xsl:value-of select="format-date(xs:date($date),'[MNn] [D], [Y]')"/>
 </xsl:function>
 
 <xsl:template match="date">
  (<i>
   <xsl:value-of select="func:getDate(node())"/>
  </i>) |
 </xsl:template>
 
 <xsl:template match="isbn">
  <b>Isbn: </b><xsl:value-of select="node()"/>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>
   <xd:p>Creates an image element.</xd:p>
  </xd:desc>
 </xd:doc>
 <xsl:template match="image">
  <img src="{@src}" alt="{@description}" width="80" height="100"/>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>
   <xd:p>Generates a title with link</xd:p>
  </xd:desc>
  <xd:param name="titleUri">The URI of the title</xd:param>
  <xd:param name="title">The value of the title</xd:param>
 </xd:doc>
 <xsl:template name="linkTitle">
  <xsl:param name="titleUri"/>
  <xsl:param name="title"/>
  <h3>
   <a href="{$titleUri}" title="{$title}">
    <xsl:value-of select="$title"/>
   </a>
  </h3>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>Creates a paragraph element.</xd:desc>
 </xd:doc>
 <xsl:template match="para">
  <p>
   <xsl:apply-templates/>
  </p>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>Creates an italic element.</xd:desc>
 </xd:doc>
 <xsl:template match="italic">
  <i>
   <xsl:apply-templates/>
  </i>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>Creates a list element.</xd:desc>
 </xd:doc>
 <xsl:template match="list">
  <ul><xsl:apply-templates select="element"/></ul>
 </xsl:template>
 
 <xd:doc>
  <xd:desc>Creates a list item element.</xd:desc>
 </xd:doc>
 <xsl:template match="element">
  <li>
   <xsl:apply-templates/>
  </li>
 </xsl:template>
</xsl:stylesheet>
